"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.formatTsType = exports.formatTsField = exports.formatTsModel = void 0;
var constants_1 = require("./constants");
function formatTsModel(_a) {
    var tsModel = _a.tsModel, idToTypeName = _a.idToTypeName;
    var id = tsModel.id, description = tsModel.description, fields = tsModel.fields;
    var typeName = idToTypeName(id);
    var comment = description.replace(/\*\//g, '');
    var heading = comment && "/* " + comment + " */\n";
    var properties = fields
        .map(function (tsField) { return formatTsField({ tsField: tsField, idToTypeName: idToTypeName }); })
        .map(function (declaration) { return "" + constants_1.INDENT + declaration + ";"; })
        .join('\n');
    return ("\n" + heading + "export interface " + typeName + " {\n" + properties + "\n}\n  ").trim();
}
exports.formatTsModel = formatTsModel;
function formatTsField(_a) {
    var tsField = _a.tsField, idToTypeName = _a.idToTypeName;
    var id = tsField.id, optional = tsField.optional, tsType = tsField.type;
    var tsTypeFormatted = formatTsType({
        tsType: tsType,
        idToTypeName: idToTypeName,
    });
    return "" + id + (optional ? '?' : '') + ": " + tsTypeFormatted;
}
exports.formatTsField = formatTsField;
function formatTsType(_a) {
    var tsType = _a.tsType, idToTypeName = _a.idToTypeName;
    switch (tsType.type) {
        case 'Entry':
            var entryType = tsType.entryIds.map(idToTypeName).sort().join(' | ') || 'any';
            return "Entry<" + entryType + ">";
        case 'array':
            var itemsType = tsType.itemsType;
            var itemsTypeFormatted = formatTsType({
                tsType: itemsType,
                idToTypeName: idToTypeName,
            });
            var itemsTypeGrouped = itemsType.type === 'enum' && itemsType.values.length > 1
                ? "(" + itemsTypeFormatted + ")"
                : itemsTypeFormatted;
            return itemsTypeGrouped + "[]";
        case 'enum':
            return tsType.values
                .map(function (value) { return (typeof value === 'string' ? "'" + value + "'" : value); })
                .join(' | ');
        default:
            return tsType.type;
    }
}
exports.formatTsType = formatTsType;
