"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.tsModelToFileName = exports.spaceModelToFileName = exports.spaceToTypesDirectoryName = exports.spaceToDirectoryName = exports.spaceToTypeName = exports.idToFileNameFactory = exports.idToTypeNameFactory = void 0;
var change_case_1 = require("change-case");
var path_1 = require("path");
var constants_1 = require("./constants");
function idToTypeNameFactory(config) {
    var typePrefix = config.typePrefix, typeSuffix = config.typeSuffix;
    var formatAffix = function (affix) {
        return affix ? change_case_1.upperCaseFirst(affix) : '';
    };
    var prefix = formatAffix(typePrefix);
    var suffix = formatAffix(typeSuffix);
    return function (id) { return "" + prefix + change_case_1.pascalCase(id) + suffix; };
}
exports.idToTypeNameFactory = idToTypeNameFactory;
function idToFileNameFactory(config) {
    var _a = config.typePrefix, typePrefix = _a === void 0 ? '' : _a, _b = config.typeSuffix, typeSuffix = _b === void 0 ? '' : _b;
    var suffix = typeSuffix ? "." + change_case_1.paramCase(typeSuffix) : '';
    return function (id) { return "" + change_case_1.paramCase(typePrefix + " " + id) + suffix + ".ts"; };
}
exports.idToFileNameFactory = idToFileNameFactory;
function spaceToTypeName(_a) {
    var spaceName = _a.spaceName, idToTypeName = _a.idToTypeName;
    return idToTypeName(spaceNameToId(spaceName));
}
exports.spaceToTypeName = spaceToTypeName;
function spaceNameToId(spaceName) {
    return spaceName + " space";
}
function spaceToDirectoryName(_a) {
    var spaceName = _a.spaceName, _b = _a.outputDirectory, outputDirectory = _b === void 0 ? './' : _b;
    return path_1.join(outputDirectory, change_case_1.paramCase(spaceNameToId(spaceName)));
}
exports.spaceToDirectoryName = spaceToDirectoryName;
function spaceToTypesDirectoryName(_a) {
    var spaceName = _a.spaceName, outputDirectory = _a.outputDirectory;
    return path_1.join(spaceToDirectoryName({ spaceName: spaceName, outputDirectory: outputDirectory }), constants_1.TYPES_DIR);
}
exports.spaceToTypesDirectoryName = spaceToTypesDirectoryName;
function spaceModelToFileName(_a) {
    var spaceName = _a.spaceName, spaceDirectory = _a.spaceDirectory, idToFileName = _a.idToFileName;
    return path_1.join(spaceDirectory, idToFileName(spaceNameToId(spaceName)));
}
exports.spaceModelToFileName = spaceModelToFileName;
function tsModelToFileName(_a) {
    var id = _a.id, spaceTypesDirectory = _a.spaceTypesDirectory, idToFileName = _a.idToFileName;
    return path_1.join(spaceTypesDirectory, idToFileName(id));
}
exports.tsModelToFileName = tsModelToFileName;
