"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.fieldToTsType = exports.transformContentType = exports.transformContentTypes = void 0;
function transformContentTypes(contentTypes) {
    var arrayIsNonEmpty = function (arr) { return arr.length > 0; };
    var tsModels = contentTypes.map(transformContentType);
    var allSpaceEntryIds = tsModels.map(function (_a) {
        var id = _a.id;
        return id;
    });
    if (!arrayIsNonEmpty(allSpaceEntryIds)) {
        return [];
    }
    return tsModels
        .map(function (tsModel) { return (__assign(__assign({}, tsModel), { fields: tsModel.fields.map(function (tsField) { return (__assign(__assign({}, tsField), { type: fillEntryIds(tsField.type, allSpaceEntryIds) })); }) })); })
        .sort(function (x, y) { return x.id.localeCompare(y.id); });
}
exports.transformContentTypes = transformContentTypes;
function transformContentType(contentType) {
    var id = contentType.sys.id, description = contentType.description, fields = contentType.fields;
    return {
        id: id,
        description: description || '',
        fields: fields.map(transformField).sort(function (x, y) { return x.id.localeCompare(y.id); }),
    };
}
exports.transformContentType = transformContentType;
function transformField(field) {
    var id = field.id, required = field.required;
    return {
        id: id,
        type: fieldToTsType(field),
        optional: !required,
    };
}
function fillEntryIds(tsType, allSpaceEntryIds) {
    if (tsType.type === 'Entry' && tsType.entryIds.length === 0) {
        return __assign(__assign({}, tsType), { entryIds: allSpaceEntryIds });
    }
    else if (tsType.type === 'array') {
        return __assign(__assign({}, tsType), { itemsType: fillEntryIds(tsType.itemsType, allSpaceEntryIds) });
    }
    return tsType;
}
function fieldToTsType(field) {
    var type = field.type, items = field.items, linkType = field.linkType, validations = field.validations;
    var tsType = mapToType(type, linkType);
    switch (tsType) {
        case 'Entry':
            return toTsEntry(validations);
        case 'array':
            return toTsArray(items);
        case 'string':
        case 'number':
            var enumType = toTsEnumIfPossible(validations);
            if (enumType) {
                return enumType;
            }
            break;
    }
    return { type: tsType };
}
exports.fieldToTsType = fieldToTsType;
// tslint:disable-next-line:cyclomatic-complexity
function mapToType(type, linkType) {
    switch (type) {
        case 'Symbol':
        case 'Text':
            return 'string';
        case 'Date':
            return 'string | Date';
        case 'Integer':
        case 'Number':
            return 'number';
        case 'Boolean':
            return 'boolean';
        case 'RichText':
            return 'Document';
        case 'Object':
            return 'object';
        case 'Location':
            return 'EntryFields.Location';
        case 'Array':
            return 'array';
        case 'Link':
            if (linkType === 'Asset') {
                return 'Asset';
            }
            if (linkType === 'Entry') {
                return 'Entry';
            }
            throw new Error("Unknown linkType " + JSON.stringify(linkType));
        default:
            throw new Error("Unknown field type " + JSON.stringify(type));
    }
}
function toTsEnumIfPossible(validations) {
    var enumValidation = validations &&
        validations.find(function (validation) { return validation.in && validation.in.length > 0; });
    if (enumValidation == null) {
        return null;
    }
    var values = enumValidation.in;
    return {
        type: 'enum',
        // tslint:disable-next-line:no-non-null-assertion
        values: values.sort(),
    };
}
function toTsEntry(validations) {
    var linkTypeValidation = validations && validations.find(function (_a) {
        var linkContentType = _a.linkContentType;
        return linkContentType;
    });
    var entryIds = (linkTypeValidation && linkTypeValidation.linkContentType) || [];
    return {
        type: 'Entry',
        entryIds: entryIds,
    };
}
function toTsArray(items) {
    if (items == null) {
        throw new Error('Array type with no items specification');
    }
    return {
        type: 'array',
        itemsType: fieldToTsType(items),
    };
}
